"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.arrayExtensions = exports.average = void 0;
const ExpressionError_1 = require("../ExpressionError");
const ObjectExtensions_1 = require("./ObjectExtensions");
function deepCompare(left, right) {
    if (left === right) {
        return true;
    }
    if (typeof left !== typeof right) {
        return false;
    }
    if (typeof left === 'number' && isNaN(left) && isNaN(right)) {
        return true;
    }
    if (['number', 'string', 'bigint', 'boolean', 'symbol'].includes(typeof left) && left !== right) {
        return false;
    }
    if (Object.keys(left).length !== Object.keys(right).length) {
        return false;
    }
    if (Array.isArray(left) !== Array.isArray(right)) {
        return false;
    }
    if (Array.isArray(left)) {
        if (left.length !== right.length) {
            return false;
        }
        return left.every((v, i) => deepCompare(v, right[i]));
    }
    for (const key in right) {
        if (left.hasOwnProperty(key) !== right.hasOwnProperty(key)) {
            return false;
        }
        else if (typeof left[key] !== typeof right[key]) {
            return false;
        }
    }
    for (const key in left) {
        if (left.hasOwnProperty(key) !== right.hasOwnProperty(key)) {
            return false;
        }
        else if (typeof left[key] !== typeof right[key]) {
            return false;
        }
        if (typeof left[key] === 'object') {
            if (left[key] !== right[key] &&
                !deepCompare(left[key], right[key])) {
                return false;
            }
        }
        else {
            if (left[key] !== right[key]) {
                return false;
            }
        }
    }
    return true;
}
function first(value) {
    return value[0];
}
function isBlank(value) {
    return value.length === 0;
}
function isPresent(value) {
    return value.length > 0;
}
function last(value) {
    return value[value.length - 1];
}
function length(value) {
    return Array.isArray(value) ? value.length : 0;
}
function pluck(value, extraArgs) {
    if (!Array.isArray(extraArgs)) {
        throw new ExpressionError_1.ExpressionError('arguments must be passed to pluck');
    }
    const fieldsToPluck = extraArgs;
    return value.map((element) => {
        const entries = Object.entries(element);
        return entries.reduce((p, c) => {
            const [key, val] = c;
            if (fieldsToPluck.includes(key)) {
                Object.assign(p, { [key]: val });
            }
            return p;
        }, {});
    });
}
function random(value) {
    const len = value === undefined ? 0 : value.length;
    return len ? value[Math.floor(Math.random() * len)] : undefined;
}
function unique(value, extraArgs) {
    if (extraArgs.length) {
        return value.reduce((l, v) => {
            if (typeof v === 'object' && v !== null && extraArgs.every((i) => i in v)) {
                const alreadySeen = l.find((i) => extraArgs.every((j) => deepCompare(i[j], v[j])));
                if (!alreadySeen) {
                    l.push(v);
                }
            }
            return l;
        }, []);
    }
    return value.reduce((l, v) => {
        if (l.findIndex((i) => deepCompare(i, v)) === -1) {
            l.push(v);
        }
        return l;
    }, []);
}
function sum(value) {
    return value.reduce((p, c) => {
        if (typeof c === 'string') {
            return p + parseFloat(c);
        }
        if (typeof c !== 'number') {
            return NaN;
        }
        return p + c;
    }, 0);
}
function min(value) {
    return Math.min(...value.map((v) => {
        if (typeof v === 'string') {
            return parseFloat(v);
        }
        if (typeof v !== 'number') {
            return NaN;
        }
        return v;
    }));
}
function max(value) {
    return Math.max(...value.map((v) => {
        if (typeof v === 'string') {
            return parseFloat(v);
        }
        if (typeof v !== 'number') {
            return NaN;
        }
        return v;
    }));
}
function average(value) {
    if (value.length === 0) {
        return 0;
    }
    return sum(value) / value.length;
}
exports.average = average;
function compact(value) {
    return value
        .filter((v) => v !== null && v !== undefined)
        .map((v) => {
        if (typeof v === 'object' && v !== null) {
            return (0, ObjectExtensions_1.compact)(v);
        }
        return v;
    });
}
function smartJoin(value, extraArgs) {
    const [keyField, valueField] = extraArgs;
    if (!keyField || !valueField || typeof keyField !== 'string' || typeof valueField !== 'string') {
        throw new ExpressionError_1.ExpressionExtensionError('smartJoin requires 2 arguments: keyField and nameField. e.g. .smartJoin("name", "value")');
    }
    return value.reduce((o, v) => {
        if (typeof v === 'object' && v !== null && keyField in v && valueField in v) {
            o[v[keyField]] = v[valueField];
        }
        return o;
    }, {});
}
function chunk(value, extraArgs) {
    const [chunkSize] = extraArgs;
    if (typeof chunkSize !== 'number') {
        throw new ExpressionError_1.ExpressionExtensionError('chunk requires 1 parameter: chunkSize. e.g. .chunk(5)');
    }
    const chunks = [];
    for (let i = 0; i < value.length; i += chunkSize) {
        chunks.push(value.slice(i, i + chunkSize));
    }
    return chunks;
}
function filter(value, extraArgs) {
    const [field, term] = extraArgs;
    if (typeof field !== 'string' && typeof field !== 'function') {
        throw new ExpressionError_1.ExpressionExtensionError('filter requires 1 or 2 arguments: (field and term), (term and [optional keepOrRemove "keep" or "remove" default "keep"] (for string arrays)), or function. e.g. .filter("type", "home") or .filter((i) => i.type === "home") or .filter("home", [optional keepOrRemove]) (for string arrays)');
    }
    if (value.every((i) => typeof i === 'string') && typeof field === 'string') {
        return value.filter((i) => term === 'remove' ? !i.includes(field) : i.includes(field));
    }
    else if (typeof field === 'string') {
        return value.filter((v) => typeof v === 'object' && v !== null && field in v && v[field] === term);
    }
    return value.filter(field);
}
function renameKeys(value, extraArgs) {
    if (extraArgs.length === 0 || extraArgs.length % 2 !== 0) {
        throw new ExpressionError_1.ExpressionExtensionError('renameKeys requires an even amount of arguments: from1, to1 [, from2, to2, ...]. e.g. .renameKeys("name", "title")');
    }
    return value.map((v) => {
        if (typeof v !== 'object' || v === null) {
            return v;
        }
        const newObj = { ...v };
        const chunkedArgs = chunk(extraArgs, [2]);
        chunkedArgs.forEach(([from, to]) => {
            if (from in newObj) {
                newObj[to] = newObj[from];
                delete newObj[from];
            }
        });
        return newObj;
    });
}
function merge(value, extraArgs) {
    const [others] = extraArgs;
    if (!Array.isArray(others)) {
        throw new ExpressionError_1.ExpressionExtensionError('merge requires 1 argument that is an array. e.g. .merge([{ id: 1, otherValue: 3 }])');
    }
    const listLength = value.length > others.length ? value.length : others.length;
    const newList = new Array(listLength);
    for (let i = 0; i < listLength; i++) {
        if (value[i] !== undefined) {
            if (typeof value[i] === 'object' && typeof others[i] === 'object') {
                newList[i] = (0, ObjectExtensions_1.merge)(value[i], [others[i]]);
            }
            else {
                newList[i] = value[i];
            }
        }
        else {
            newList[i] = others[i];
        }
    }
    return newList;
}
function union(value, extraArgs) {
    const [others] = extraArgs;
    if (!Array.isArray(others)) {
        throw new ExpressionError_1.ExpressionExtensionError('union requires 1 argument that is an array. e.g. .union([1, 2, 3, 4])');
    }
    const newArr = Array.from(value);
    for (const v of others) {
        if (newArr.findIndex((w) => deepCompare(w, v)) === -1) {
            newArr.push(v);
        }
    }
    return unique(newArr, []);
}
function difference(value, extraArgs) {
    const [others] = extraArgs;
    if (!Array.isArray(others)) {
        throw new ExpressionError_1.ExpressionExtensionError('difference requires 1 argument that is an array. e.g. .difference([1, 2, 3, 4])');
    }
    const newArr = [];
    for (const v of value) {
        if (others.findIndex((w) => deepCompare(w, v)) === -1) {
            newArr.push(v);
        }
    }
    return unique(newArr, []);
}
function intersection(value, extraArgs) {
    const [others] = extraArgs;
    if (!Array.isArray(others)) {
        throw new ExpressionError_1.ExpressionExtensionError('difference requires 1 argument that is an array. e.g. .difference([1, 2, 3, 4])');
    }
    const newArr = [];
    for (const v of value) {
        if (others.findIndex((w) => deepCompare(w, v)) !== -1) {
            newArr.push(v);
        }
    }
    for (const v of others) {
        if (value.findIndex((w) => deepCompare(w, v)) !== -1) {
            newArr.push(v);
        }
    }
    return unique(newArr, []);
}
exports.arrayExtensions = {
    typeName: 'Array',
    functions: {
        count: length,
        duplicates: unique,
        filter,
        first,
        last,
        length,
        pluck,
        unique,
        random,
        randomItem: random,
        remove: unique,
        size: length,
        sum,
        min,
        max,
        average,
        isPresent,
        isBlank,
        compact,
        smartJoin,
        chunk,
        renameKeys,
        merge,
        union,
        difference,
        intersection,
    },
};
//# sourceMappingURL=ArrayExtensions.js.map