"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dateExtensions = void 0;
const luxon_1 = require("luxon");
const DURATION_MAP = {
    day: 'days',
    month: 'months',
    year: 'years',
    week: 'weeks',
    hour: 'hours',
    minute: 'minutes',
    second: 'seconds',
    millisecond: 'milliseconds',
    ms: 'milliseconds',
    sec: 'seconds',
    secs: 'seconds',
    hr: 'hours',
    hrs: 'hours',
    min: 'minutes',
    mins: 'minutes',
};
const DATETIMEUNIT_MAP = {
    days: 'day',
    months: 'month',
    years: 'year',
    hours: 'hour',
    minutes: 'minute',
    seconds: 'second',
    milliseconds: 'millisecond',
    hrs: 'hour',
    hr: 'hour',
    mins: 'minute',
    min: 'minute',
    secs: 'second',
    sec: 'second',
    ms: 'millisecond',
};
function isDateTime(date) {
    if (date) {
        return luxon_1.DateTime.isDateTime(date);
    }
    return false;
}
function generateDurationObject(durationValue, unit) {
    const convertedUnit = DURATION_MAP[unit] || unit;
    return { [`${convertedUnit}`]: durationValue };
}
function beginningOf(date, extraArgs) {
    const [unit = 'week'] = extraArgs;
    if (isDateTime(date)) {
        return date.startOf(DATETIMEUNIT_MAP[unit] || unit).toJSDate();
    }
    let datetime = luxon_1.DateTime.fromJSDate(date);
    if (date.getTimezoneOffset() === 0) {
        datetime = datetime.setZone('UTC');
    }
    return datetime.startOf(DATETIMEUNIT_MAP[unit] || unit).toJSDate();
}
function endOfMonth(date) {
    if (isDateTime(date)) {
        return date.endOf('month').toJSDate();
    }
    return luxon_1.DateTime.fromJSDate(date).endOf('month').toJSDate();
}
function extract(inputDate, extraArgs) {
    const [part] = extraArgs;
    let date = inputDate;
    if (isDateTime(date)) {
        date = date.toJSDate();
    }
    if (part === 'yearDayNumber') {
        const firstDayOfTheYear = new Date(date.getFullYear(), 0, 0);
        const diff = date.getTime() -
            firstDayOfTheYear.getTime() +
            (firstDayOfTheYear.getTimezoneOffset() - date.getTimezoneOffset()) * 60 * 1000;
        return Math.floor(diff / (1000 * 60 * 60 * 24));
    }
    return luxon_1.DateTime.fromJSDate(date).get(DATETIMEUNIT_MAP[part] || part);
}
function format(date, extraArgs) {
    const [dateFormat, localeOpts = {}] = extraArgs;
    if (isDateTime(date)) {
        return date.toFormat(dateFormat, { ...localeOpts });
    }
    return luxon_1.DateTime.fromJSDate(date).toFormat(dateFormat, { ...localeOpts });
}
function isBetween(date, extraArgs) {
    const [first, second] = extraArgs;
    const firstDate = new Date(first);
    const secondDate = new Date(second);
    if (firstDate > secondDate) {
        return secondDate < date && date < firstDate;
    }
    return secondDate > date && date > firstDate;
}
function isDst(date) {
    if (isDateTime(date)) {
        return date.isInDST;
    }
    return luxon_1.DateTime.fromJSDate(date).isInDST;
}
function isInLast(date, extraArgs) {
    const [durationValue = 0, unit = 'minutes'] = extraArgs;
    const dateInThePast = luxon_1.DateTime.now().minus(generateDurationObject(durationValue, unit));
    let thisDate = date;
    if (!isDateTime(thisDate)) {
        thisDate = luxon_1.DateTime.fromJSDate(thisDate);
    }
    return dateInThePast <= thisDate && thisDate <= luxon_1.DateTime.now();
}
function isWeekend(date) {
    let DAYS;
    (function (DAYS) {
        DAYS[DAYS["saturday"] = 6] = "saturday";
        DAYS[DAYS["sunday"] = 7] = "sunday";
    })(DAYS || (DAYS = {}));
    if (isDateTime(date)) {
        return [DAYS.saturday, DAYS.sunday].includes(date.weekday);
    }
    return [DAYS.saturday, DAYS.sunday].includes(luxon_1.DateTime.fromJSDate(date).weekday);
}
function minus(date, extraArgs) {
    if (isDateTime(date) && extraArgs.length === 1) {
        return date.minus(extraArgs[0]);
    }
    const [durationValue = 0, unit = 'minutes'] = extraArgs;
    if (isDateTime(date)) {
        return date.minus(generateDurationObject(durationValue, unit)).toJSDate();
    }
    return luxon_1.DateTime.fromJSDate(date).minus(generateDurationObject(durationValue, unit)).toJSDate();
}
function plus(date, extraArgs) {
    if (isDateTime(date) && extraArgs.length === 1) {
        return date.plus(extraArgs[0]);
    }
    const [durationValue = 0, unit = 'minutes'] = extraArgs;
    if (isDateTime(date)) {
        return date.plus(generateDurationObject(durationValue, unit)).toJSDate();
    }
    return luxon_1.DateTime.fromJSDate(date).plus(generateDurationObject(durationValue, unit)).toJSDate();
}
function toLocaleString(date, extraArgs) {
    const [locale, dateFormat = { timeStyle: 'short', dateStyle: 'short' }] = extraArgs;
    if (isDateTime(date)) {
        return date.toLocaleString(dateFormat, { locale });
    }
    return luxon_1.DateTime.fromJSDate(date).toLocaleString(dateFormat, { locale });
}
function toTimeFromNow(date) {
    let diffObj;
    if (isDateTime(date)) {
        diffObj = date.diffNow();
    }
    else {
        diffObj = luxon_1.DateTime.fromJSDate(date).diffNow();
    }
    const as = (unit) => {
        return Math.round(Math.abs(diffObj.as(unit)));
    };
    if (as('years')) {
        return `${as('years')} years ago`;
    }
    if (as('months')) {
        return `${as('months')} months ago`;
    }
    if (as('weeks')) {
        return `${as('weeks')} weeks ago`;
    }
    if (as('days')) {
        return `${as('days')} days ago`;
    }
    if (as('hours')) {
        return `${as('hours')} hours ago`;
    }
    if (as('minutes')) {
        return `${as('minutes')} minutes ago`;
    }
    if (as('seconds') && as('seconds') > 10) {
        return `${as('seconds')} seconds ago`;
    }
    return 'just now';
}
function timeTo(date, extraArgs) {
    const [diff = new Date().toISOString(), unit = 'seconds'] = extraArgs;
    const diffDate = new Date(diff);
    if (isDateTime(date)) {
        return date.diff(luxon_1.DateTime.fromJSDate(diffDate), DURATION_MAP[unit] || unit);
    }
    return luxon_1.DateTime.fromJSDate(date).diff(luxon_1.DateTime.fromJSDate(diffDate), DURATION_MAP[unit] || unit);
}
function toDate(date) {
    if (isDateTime(date)) {
        return date.set({ hour: 0, minute: 0, second: 0, millisecond: 0 }).toJSDate();
    }
    let datetime = luxon_1.DateTime.fromJSDate(date);
    if (date.getTimezoneOffset() === 0) {
        datetime = datetime.setZone('UTC');
    }
    return datetime.set({ hour: 0, minute: 0, second: 0, millisecond: 0 }).toJSDate();
}
exports.dateExtensions = {
    typeName: 'Date',
    functions: {
        beginningOf,
        endOfMonth,
        extract,
        isBetween,
        isDst,
        isInLast,
        isWeekend,
        minus,
        plus,
        toTimeFromNow,
        timeTo,
        format,
        toLocaleString,
        toDate,
    },
};
//# sourceMappingURL=DateExtensions.js.map